@@ -23,6 +23,10 @@ BEOPLAY_URL_PAUSE = 'BeoZone/Zone/Stream/Pause'
 BEOPLAY_URL_STOP = 'BeoZone/Zone/Stream/Stop'
 BEOPLAY_URL_FORWARD = 'BeoZone/Zone/Stream/Forward'
 BEOPLAY_URL_BACKWARD = 'BeoZone/Zone/Stream/Backward'
+BEOPLAY_URL_STEPUP = 'BeoZone/Zone/List/StepUp'
+BEOPLAY_URL_STEPDOWN = 'BeoZone/Zone/List/StepDown'
+BEOPLAY_URL_SHUFFLE = 'BeoZone/Zone/List/Shuffle'
+BEOPLAY_URL_REPEAT = 'BeoZone/Zone/List/Repeat'
 BEOPLAY_URL_STANDBY = 'BeoDevice/powerManagement/standby'
 BEOPLAY_URL_SET_SOURCE = 'BeoZone/Zone/ActiveSources'
 BEOPLAY_URL_SET_STAND = 'BeoZone/Zone/Stand/Active'
